/*-
 * Copyright (c) 2000
 * Tatsuya Kudoh(CDR/TK),ROYALPANDA.    All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include<stdio.h>

unsigned char Buf[1048576];


int print_list(int i)
{
	int addr;
	int off;
	unsigned char *p,*np;;

	addr = (unsigned)Buf[i*4]
		| (unsigned)Buf[i*4+1] << 8
		| (unsigned)Buf[i*4+2] << 16
		| (unsigned)Buf[i*4+3] << 24;
	if( addr == 0 )
		return;

	np = Buf+addr;
	do{
		p = np;
		off = (unsigned)p[0] | (unsigned)p[1] << 8;
		np += off;

		p+=2;
		printf("%s:",p);
		while( *p )
			p++;
		if( *p++ )
			return -1;
		if( *p++ )
			return -1;

		while( *p ){
			printf("%s ",p);
			while( *p++ )
				;
		}
		printf("\n");
	}while(off);
	printf("\n");
	return 0;
}
		






int main( void )
{
	unsigned char *p;
	int i;

	p = Buf;
	while( (i = getchar()) != EOF )
		*p++ = i;

	for( i = 0 ; i < 256 ; i++ ){
		printf("hash %d\n",i);
		if( print_list(i) < 0 )
			printf("\n\nerror\n");
	}
	return 0;
}
